"""Change history tracking for the agents SDK.

This module provides audit trail functionality to track who changed what,
when, and why, with revision history, diff capabilities, and revert functionality.
"""

from __future__ import annotations

import json
from dataclasses import dataclass
from datetime import datetime, timezone
from pathlib import Path
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from better_notion._sdk.client import NotionClient


@dataclass
class PropertyChange:
    """Represents a single property change.

    Attributes:
        property_name: Name of the property that changed
        from_value: Previous value (None if property was added)
        to_value: New value (None if property was removed)
    """

    property_name: str
    from_value: Any
    to_value: Any

    def to_dict(self) -> dict[str, Any]:
        """Convert property change to dictionary.

        Returns:
            Dictionary with property, from, and to values
        """
        return {
            "property": self.property_name,
            "from": self.from_value,
            "to": self.to_value,
        }


@dataclass
class Revision:
    """Represents a single revision of an entity.

    Attributes:
        revision_id: Sequential revision number (starting at 1)
        timestamp: When the change was made
        author: Who made the change
        changes: List of property changes in this revision
        reason: Optional explanation for the change
    """

    revision_id: int
    timestamp: datetime
    author: str
    changes: list[PropertyChange]
    reason: str | None

    def to_dict(self) -> dict[str, Any]:
        """Convert revision to dictionary.

        Returns:
            Dictionary with all revision data
        """
        return {
            "revision_id": self.revision_id,
            "timestamp": self.timestamp.isoformat(),
            "author": self.author,
            "changes": [c.to_dict() for c in self.changes],
            "reason": self.reason,
        }


class HistoryStorage:
    """Storage backend for history data.

    Supports both local file storage (JSONL) and optional Notion database storage.
    """

    def __init__(self, client: NotionClient) -> None:
        """Initialize the HistoryStorage.

        Args:
            client: Notion API client
        """
        self._client = client
        self._history_root = Path(".notion_history")

    async def save_revision(
        self,
        entity_id: str,
        entity_type: str,
        revision: Revision,
    ) -> None:
        """Save a revision to storage.

        Args:
            entity_id: Entity ID
            entity_type: Entity type (e.g., "task", "idea")
            revision: Revision to save
        """
        # Try Notion storage first if configured
        database_id = self._client.workspace_config.get("Change_History")
        if database_id:
            try:
                await self._save_to_notion(database_id, entity_id, entity_type, revision)
                return
            except Exception:
                # Fall back to local storage on error
                pass

        # Default to local storage
        await self._save_to_local(entity_id, entity_type, revision)

    async def _save_to_notion(
        self,
        database_id: str,
        entity_id: str,
        entity_type: str,
        revision: Revision,
    ) -> None:
        """Save revision to Notion database.

        Args:
            database_id: Change History database ID
            entity_id: Entity ID
            entity_type: Entity type
            revision: Revision to save
        """
        from better_notion._sdk.models.page import Page

        # Create a page in the Change History database
        changes_text = "\n".join([
            f"{c.property_name}: {c.from_value} → {c.to_value}"
            for c in revision.changes
        ])

        properties = {
            "Entity ID": [{"type": "text", "text": {"content": entity_id}}],
            "Entity Type": {"select": {"name": entity_type}},
            "Revision": {"number": revision.revision_id},
            "Author": [{"type": "text", "text": {"content": revision.author}}],
            "Timestamp": {"date": {"start": revision.timestamp.isoformat()}},
            "Changes": [
                {
                    "type": "text",
                    "text": {
                        "content": changes_text,
                    },
                }
            ],
        }

        if revision.reason:
            properties["Reason"] = [
                {
                    "type": "text",
                    "text": {
                        "content": revision.reason,
                    },
                }
            ]

        await Page.create(
            client=self._client,
            database_id=database_id,
            title=f"{entity_type}:{entity_id} - r{revision.revision_id}",
            properties=properties,
        )

    async def _save_to_local(
        self,
        entity_id: str,
        entity_type: str,
        revision: Revision,
    ) -> None:
        """Save revision to local file storage.

        Args:
            entity_id: Entity ID
            entity_type: Entity type
            revision: Revision to save
        """
        # Create history directory
        history_dir = self._history_root / entity_type
        history_dir.mkdir(parents=True, exist_ok=True)

        # Save to JSONL file
        history_file = history_dir / f"{entity_id}.jsonl"

        # Append revision
        with open(history_file, "a", encoding="utf-8") as f:
            f.write(json.dumps(revision.to_dict()) + "\n")

    async def get_revisions(self, entity_id: str, entity_type: str | None = None) -> list[Revision]:
        """Get all revisions for an entity.

        Args:
            entity_id: Entity ID
            entity_type: Optional entity type to narrow search

        Returns:
            List of revisions sorted by revision_id
        """
        # Try local storage first
        revisions = await self._get_from_local(entity_id, entity_type)

        if not revisions:
            # Try Notion storage (not implemented in v1)
            revisions = await self._get_from_notion(entity_id)

        return revisions

    async def _get_from_local(
        self,
        entity_id: str,
        entity_type: str | None = None,
    ) -> list[Revision]:
        """Get revisions from local storage.

        Args:
            entity_id: Entity ID
            entity_type: Optional entity type to narrow search

        Returns:
            List of revisions
        """
        if not self._history_root.exists():
            return []

        # If entity_type is specified, look in that directory
        if entity_type:
            history_file = self._history_root / entity_type / f"{entity_id}.jsonl"
            if history_file.exists():
                return self._load_revisions_from_file(history_file)
            return []

        # Otherwise, search all entity type directories
        for entity_dir in self._history_root.iterdir():
            if entity_dir.is_dir():
                history_file = entity_dir / f"{entity_id}.jsonl"
                if history_file.exists():
                    return self._load_revisions_from_file(history_file)

        return []

    def _load_revisions_from_file(self, history_file: Path) -> list[Revision]:
        """Load revisions from a JSONL file.

        Args:
            history_file: Path to the JSONL file

        Returns:
            List of revisions
        """
        revisions = []
        with open(history_file, encoding="utf-8") as f:
            for line in f:
                data = json.loads(line)
                revisions.append(
                    Revision(
                        revision_id=data["revision_id"],
                        timestamp=datetime.fromisoformat(data["timestamp"]),
                        author=data["author"],
                        changes=[
                            PropertyChange(
                                property_name=c["property"],
                                from_value=c.get("from"),
                                to_value=c.get("to"),
                            )
                            for c in data["changes"]
                        ],
                        reason=data.get("reason"),
                    )
                )

        # Sort by revision_id
        revisions.sort(key=lambda r: r.revision_id)
        return revisions

    async def _get_from_notion(self, entity_id: str) -> list[Revision]:
        """Get revisions from Notion storage.

        Args:
            entity_id: Entity ID

        Returns:
            List of revisions (empty in v1 - Notion storage not implemented)
        """
        # Not implemented in v1
        # Would query Change_History database and filter by Entity ID
        return []

    async def get_audit_log(
        self,
        project_id: str | None = None,
        days: int = 7,
        entity_type: str | None = None,
    ) -> list[dict[str, Any]]:
        """Get audit log for changes.

        Args:
            project_id: Optional project ID to filter by
            days: Number of days to look back
            entity_type: Optional entity type filter

        Returns:
            List of change records with metadata
        """
        # In v1, scan local storage for recent changes
        cutoff_time = datetime.now(timezone.utc).replace(hour=0, minute=0, second=0, microsecond=0)
        cutoff_time = cutoff_time.replace(day=cutoff_time.day - days)

        changes = []

        if not self._history_root.exists():
            return changes

        # Scan entity type directories
        for entity_dir in self._history_root.iterdir():
            if not entity_dir.is_dir():
                continue

            # Filter by entity type if specified
            if entity_type and entity_dir.name != entity_type:
                continue

            for history_file in entity_dir.glob("*.jsonl"):
                file_changes = self._load_recent_changes_from_file(history_file, cutoff_time)
                changes.extend(file_changes)

        # Sort by timestamp (newest first)
        changes.sort(key=lambda c: c["timestamp"], reverse=True)

        return changes

    def _load_recent_changes_from_file(
        self,
        history_file: Path,
        cutoff_time: datetime,
    ) -> list[dict[str, Any]]:
        """Load recent changes from a JSONL file.

        Args:
            history_file: Path to the JSONL file
            cutoff_time: Only include changes after this time

        Returns:
            List of change records
        """
        changes = []
        entity_id = history_file.stem  # Filename is entity ID
        entity_type = history_file.parent.name

        with open(history_file, encoding="utf-8") as f:
            for line in f:
                data = json.loads(line)
                timestamp = datetime.fromisoformat(data["timestamp"])

                if timestamp >= cutoff_time:
                    changes.append(
                        {
                            "timestamp": timestamp,
                            "author": data["author"],
                            "entity_type": entity_type,
                            "entity_id": entity_id,
                            "revision_id": data["revision_id"],
                            "changes": data["changes"],
                            "reason": data.get("reason"),
                        }
                    )

        return changes


class HistoryTracker:
    """Track and manage change history for entities.

    Provides methods to record changes, retrieve history, compare revisions,
    and generate audit logs.
    """

    def __init__(self, client: NotionClient) -> None:
        """Initialize the HistoryTracker.

        Args:
            client: Notion API client
        """
        self._client = client
        self._storage = HistoryStorage(client)

    async def record_creation(
        self,
        entity_id: str,
        entity_type: str,
        author: str,
        properties: dict[str, Any],
    ) -> None:
        """Record entity creation.

        Args:
            entity_id: Entity ID
            entity_type: Entity type (e.g., "task", "idea")
            author: Who created the entity
            properties: Initial property values
        """
        changes = [
            PropertyChange(
                property_name=prop,
                from_value=None,
                to_value=self._format_value(value),
            )
            for prop, value in properties.items()
        ]

        revision = Revision(
            revision_id=1,
            timestamp=datetime.now(timezone.utc),
            author=author,
            changes=changes,
            reason="Initial creation",
        )

        await self._storage.save_revision(entity_id, entity_type, revision)

    async def record_update(
        self,
        entity_id: str,
        entity_type: str,
        author: str,
        old_properties: dict[str, Any],
        new_properties: dict[str, Any],
        reason: str | None = None,
    ) -> None:
        """Record entity update.

        Args:
            entity_id: Entity ID
            entity_type: Entity type
            author: Who made the change
            old_properties: Property values before change
            new_properties: Property values after change
            reason: Optional explanation for the change
        """
        changes = []

        # Find changed properties
        all_keys = set(old_properties.keys()) | set(new_properties.keys())

        for key in all_keys:
            old_val = old_properties.get(key)
            new_val = new_properties.get(key)

            if old_val != new_val:
                changes.append(
                    PropertyChange(
                        property_name=key,
                        from_value=self._format_value(old_val),
                        to_value=self._format_value(new_val),
                    )
                )

        if not changes:
            return  # No changes to record

        # Get next revision number
        current_revisions = await self._storage.get_revisions(entity_id, entity_type)
        next_revision_id = len(current_revisions) + 1

        revision = Revision(
            revision_id=next_revision_id,
            timestamp=datetime.now(timezone.utc),
            author=author,
            changes=changes,
            reason=reason,
        )

        await self._storage.save_revision(entity_id, entity_type, revision)

    async def get_history(self, entity_id: str, entity_type: str) -> list[Revision]:
        """Get full history for an entity.

        Args:
            entity_id: Entity ID
            entity_type: Entity type

        Returns:
            List of revisions sorted by revision_id
        """
        return await self._storage.get_revisions(entity_id, entity_type)

    async def get_revision(
        self,
        entity_id: str,
        entity_type: str,
        revision_id: int,
    ) -> dict[str, Any] | None:
        """Get entity state at a specific revision.

        Args:
            entity_id: Entity ID
            entity_type: Entity type
            revision_id: Revision number to retrieve

        Returns:
            Dictionary of property values at that revision, or None if not found
        """
        revisions = await self._storage.get_revisions(entity_id, entity_type)

        if not revisions or revision_id < 1 or revision_id > len(revisions):
            return None

        # Reconstruct state by applying changes up to revision
        state: dict[str, Any] = {}
        for revision in revisions[:revision_id]:
            for change in revision.changes:
                state[change.property_name] = change.to_value

        return state

    async def compare_revisions(
        self,
        entity_id: str,
        entity_type: str,
        from_revision: int,
        to_revision: int,
    ) -> dict[str, Any]:
        """Compare two revisions.

        Args:
            entity_id: Entity ID
            entity_type: Entity type
            from_revision: Starting revision number
            to_revision: Ending revision number

        Returns:
            Dictionary with comparison results and changes
        """
        revisions = await self._storage.get_revisions(entity_id, entity_type)

        if not revisions:
            raise ValueError(f"No revisions found for {entity_type}:{entity_id}")

        if from_revision < 1 or from_revision > len(revisions):
            raise ValueError(f"Invalid from_revision: {from_revision}")

        if to_revision < 1 or to_revision > len(revisions):
            raise ValueError(f"Invalid to_revision: {to_revision}")

        # Get changes between revisions
        changes = []
        for revision in revisions[from_revision:to_revision]:
            changes.extend(revision.changes)

        return {
            "entity_id": entity_id,
            "entity_type": entity_type,
            "comparing": {
                "from_revision": from_revision,
                "to_revision": to_revision,
            },
            "changes": [c.to_dict() for c in changes],
        }

    async def get_audit_log(
        self,
        project_id: str | None = None,
        days: int = 7,
        entity_type: str | None = None,
    ) -> dict[str, Any]:
        """Get audit log for changes.

        Args:
            project_id: Optional project ID to filter by (not implemented in v1)
            days: Number of days to look back
            entity_type: Optional entity type filter

        Returns:
            Dictionary with changes and summary statistics
        """
        changes = await self._storage.get_audit_log(project_id, days, entity_type)

        # Build summary statistics
        by_author: dict[str, int] = {}
        by_action: dict[str, int] = {"created": 0, "updated": 0}
        by_type: dict[str, int] = {}

        for change in changes:
            # Count by author
            author = change["author"]
            by_author[author] = by_author.get(author, 0) + 1

            # Count by action (creation has revision_id == 1)
            if change["revision_id"] == 1:
                by_action["created"] += 1
            else:
                by_action["updated"] += 1

            # Count by type
            etype = change["entity_type"]
            by_type[etype] = by_type.get(etype, 0) + 1

        return {
            "changes": changes,
            "summary": {
                "total_changes": len(changes),
                "by_author": by_author,
                "by_action": by_action,
                "by_type": by_type,
            },
        }

    def _format_value(self, value: Any) -> Any:
        """Format a value for storage.

        Args:
            value: Value to format

        Returns:
            Formatted value suitable for JSON serialization
        """
        if value is None:
            return None
        elif isinstance(value, dict):
            # Handle complex types like select, relation, etc.
            if "name" in value:
                return value["name"]
            elif "title" in value:
                # Extract text content from title
                title_list = value["title"]
                if title_list and len(title_list) > 0:
                    return title_list[0].get("text", {}).get("content", "")
                return ""
            elif "rich_text" in value:
                # Extract text content from rich_text
                text_list = value["rich_text"]
                if text_list and len(text_list) > 0:
                    return text_list[0].get("text", {}).get("content", "")
                return ""
            elif "date" in value:
                date_obj = value["date"]
                if date_obj:
                    return date_obj.get("start", "")
                return ""
            else:
                return str(value)
        elif isinstance(value, list):
            # Handle multi-select or relations
            if len(value) > 0 and isinstance(value[0], dict):
                return [v.get("name", str(v)) for v in value]
            return value
        else:
            return value



"""Workspace metadata management for agents plugin.

This module provides functionality to manage workspace metadata,
including detecting duplicate workspaces and storing workspace information.
"""

import json
import logging
from datetime import datetime, timezone
from pathlib import Path
from typing import Any, Optional
from uuid import uuid4

from better_notion._sdk.client import NotionClient
from better_notion._sdk.models.page import Page

logger = logging.getLogger(__name__)


class WorkspaceMetadata:
    """Manage workspace metadata for duplicate detection."""

    # Property names used in Notion page
    PROP_WORKSPACE_ID = "agents_workspace_id"
    PROP_WORKSPACE_NAME = "agents_workspace_name"
    PROP_INITIALIZED_AT = "agents_workspace_created"
    PROP_VERSION = "agents_workspace_version"
    PROP_DATABASE_IDS = "agents_workspace_databases"

    @staticmethod
    def generate_workspace_id() -> str:
        """Generate a unique workspace ID.

        Returns:
            Unique workspace ID (UUID without hyphens for compactness)
        """
        return str(uuid4()).replace("-", "")

    @staticmethod
    def extract_metadata_from_page(page: Page) -> dict[str, Any]:
        """Extract workspace metadata from page properties.

        Args:
            page: Page object to extract metadata from

        Returns:
            Dict with workspace metadata (empty if not initialized)

        Example:
            >>> metadata = WorkspaceMetadata.extract_metadata_from_page(page)
            >>> if metadata.get("workspace_id"):
            ...     print(f"Workspace: {metadata['workspace_name']}")
        """
        metadata = {}

        # Try to get workspace ID from icon (stored as emoji)
        # Notion doesn't support custom properties, so we use creative approaches

        # Check if page icon contains our metadata marker
        icon = page.icon
        if icon and icon.startswith("🤖"):
            metadata["is_agents_workspace"] = True
            # Could encode data in icon, but keep it simple for now

        return metadata

    @staticmethod
    async def detect_workspace(
        page: Page,
        client: NotionClient
    ) -> dict[str, Any] | None:
        """Detect if page already has an agents workspace.

        Args:
            page: Parent page to check
            client: NotionClient instance

        Returns:
            Workspace metadata dict if found, None otherwise
        """
        # Method 1: Check local config file first (fastest)
        config_path = Path.home() / ".notion" / "workspace.json"
        if config_path.exists():
            try:
                with open(config_path, "r", encoding="utf-8") as f:
                    config = json.load(f)
                    # Normalize page IDs for comparison (config may have dashes or not)
                    config_parent = config.get("parent_page", "").replace("-", "")
                    page_id_normalized = page.id.replace("-", "")
                    # Only use config if it matches this page
                    if config_parent == page_id_normalized:
                        # Verify at least one database still exists
                        database_ids = config.get("database_ids", {})
                        if database_ids:
                            # Check if Tasks database still exists (as a verification)
                            tasks_db_id = database_ids.get("tasks")
                            if tasks_db_id:
                                try:
                                    await client.databases.get(tasks_db_id)
                                    logger.info(f"Workspace detected from config file for page {page.id}")
                                    return {
                                        "workspace_id": config.get("workspace_id"),
                                        "workspace_name": config.get("workspace_name"),
                                        "initialized_at": config.get("initialized_at"),
                                        "database_ids": database_ids,
                                        "detection_method": "config_file",
                                        "databases_count": len(database_ids)
                                    }
                                except Exception:
                                    logger.debug("Tasks database from config not found, falling back to scan")
            except Exception as e:
                logger.debug(f"Could not load local config: {e}")

        # Method 2: Scan for databases with expected names (fallback)
        expected_databases = [
            "Organizations",
            "Tags",
            "Projects",
            "Versions",
            "Tasks",
            "Ideas",
            "Work Issues",
            "Incidents"
        ]

        try:
            # Search for databases in this page
            results = await client.search(
                query="",
                filter={"value": "database", "property": "object"}
            )

            databases_found = []
            database_ids = {}

            for result in results:
                if hasattr(result, 'title') and result.title in expected_databases:
                    databases_found.append(result.title)
                    # Extract the key name (lowercase with underscores)
                    key = result.title.lower().replace(" ", "_")
                    database_ids[key] = result.id

            # Check if we have at least 5 of the expected databases
            matches = len(databases_found)
            if matches >= 5:
                logger.info(f"Detected existing workspace with {matches}/{len(expected_databases)} databases in page {page.id}")

                return {
                    "workspace_id": None,  # Not available from scan
                    "workspace_name": None,
                    "initialized_at": None,
                    "database_ids": database_ids,
                    "detection_method": "database_scan",
                    "databases_count": matches
                }
        except Exception as e:
            logger.debug(f"Could not scan for databases: {e}")

        return None

    @staticmethod
    def save_workspace_config(
        page_id: str,
        workspace_id: str,
        workspace_name: str,
        database_ids: dict[str, str],
        path: Optional[Path] = None
    ) -> Path:
        """Save workspace configuration to local file.

        Args:
            page_id: Parent page ID
            workspace_id: Unique workspace ID
            workspace_name: Workspace name
            database_ids: Dict of database name to ID
            path: Optional custom path for config file

        Returns:
            Path to saved config file
        """
        if path is None:
            path = Path.home() / ".notion" / "workspace.json"

        path.parent.mkdir(parents=True, exist_ok=True)

        config = {
            "workspace_id": workspace_id,
            "workspace_name": workspace_name,
            "parent_page": page_id,
            "initialized_at": datetime.now(timezone.utc).isoformat(),
            "version": "1.5.4",  # Track version for migrations
            "database_ids": database_ids
        }

        with open(path, "w", encoding="utf-8") as f:
            json.dump(config, f, indent=2)

        logger.info(f"Saved workspace config to {path}")
        return path



"""Schema registry for plugin documentation.

This module provides a central registry where plugins can register
their documentation schemas for AI agent consumption.
"""

import logging
from typing import Any

from better_notion._cli.docs.base import Schema

logger = logging.getLogger(__name__)


class SchemaRegistry:
    """Registry for plugin documentation schemas.

    This class maintains a collection of plugin schemas that
    AI agents can query to understand the system.

    Example:
        >>> # Register a plugin schema
        >>> SchemaRegistry.register("agents", AGENTS_SCHEMA)
        >>>
        >>> # Query for a plugin schema
        >>> schema = SchemaRegistry.get("agents")
        >>>
        >>> # List all available schemas
        >>> all_schemas = SchemaRegistry.list_all()
    """

    _schemas: dict[str, Schema] = {}

    @classmethod
    def register(cls, plugin_name: str, schema: Schema) -> None:
        """Register a plugin schema.

        Args:
            plugin_name: Name of the plugin (e.g., "agents")
            schema: Schema object containing plugin documentation
        """
        cls._schemas[plugin_name] = schema
        logger.info(f"Registered schema for plugin: {plugin_name}")

    @classmethod
    def get(cls, plugin_name: str) -> Schema | None:
        """Get a plugin schema by name.

        Args:
            plugin_name: Name of the plugin

        Returns:
            Schema object if found, None otherwise
        """
        return cls._schemas.get(plugin_name)

    @classmethod
    def list_all(cls) -> dict[str, Schema]:
        """List all registered plugin schemas.

        Returns:
            Dict mapping plugin names to Schema objects
        """
        return cls._schemas.copy()

    @classmethod
    def get_all_names(cls) -> list[str]:
        """Get list of all registered plugin names.

        Returns:
            List of plugin names
        """
        return list(cls._schemas.keys())

    @classmethod
    def clear(cls) -> None:
        """Clear all registered schemas.

        This is primarily useful for testing.
        """
        cls._schemas.clear()
        logger.debug("Cleared schema registry")



"""Personal workspace initialization."""

from __future__ import annotations

import json
import uuid
from datetime import datetime
from pathlib import Path

from better_notion._sdk.client import NotionClient


class PersonalWorkspaceInitializer:
    """Initialize personal workspace with all databases and relationships."""

    EXPECTED_DATABASES = [
        "Domains",
        "Tags",
        "Projects",
        "Tasks",
        "Routines",
        "Agenda",
    ]

    def __init__(self, client: NotionClient):
        """Initialize workspace initializer.

        Args:
            client: Notion client instance
        """
        self.client = client
        self._database_ids: dict[str, str] = {}
        self._parent_page_id: str = ""
        self._workspace_id: str = ""
        self._workspace_name: str = ""

    async def initialize_workspace(
        self,
        parent_page_id: str,
        workspace_name: str = "Personal Workspace",
        skip_detection: bool = False,
    ) -> dict[str, str]:
        """Create all personal databases with relationships.

        Args:
            parent_page_id: Parent page ID where databases will be created
            workspace_name: Name for the workspace
            skip_detection: If True, skip workspace detection

        Returns:
            Dictionary mapping database names to their IDs
        """
        from better_notion._sdk.models.database import Database

        self._parent_page_id = parent_page_id
        self._workspace_name = workspace_name
        self._workspace_id = f"personal-{uuid.uuid4().hex[:8]}"

        # Helper function to find existing database
        async def find_existing_database(title: str) -> str | None:
            """Check if a database with given title already exists in the parent page."""
            from better_notion._api.collections import DatabaseCollection

            # Query for child databases in the parent page
            # We need to search through children to find databases
            try:
                # Get all children blocks
                response = await self.client._api._request(
                    "GET",
                    f"/blocks/{parent_page_id}/children"
                )

                for child in response.get("results", []):
                    if child.get("type") == "database":
                        # Get database details to check title
                        db_response = await self.client._api._request(
                            "GET",
                            f"/databases/{child['id']}"
                        )
                        db_title = db_response.get("title", [])
                        if db_title and len(db_title) > 0:
                            title_text = db_title[0].get("plain_text", "")
                            if title_text == title:
                                return child["id"]
            except Exception:
                # If search fails, proceed with creation
                pass

            return None

        # Step 1: Create Domains database (or reuse existing)
        existing_domains_id = await find_existing_database("Domains")
        if existing_domains_id:
            print(f"[DEBUG] Reusing existing Domains database: {existing_domains_id}")
            domain_id = existing_domains_id
        else:
            print(f"[DEBUG] Creating new Domains database")
            domain_db = await self._create_database(
                parent_page_id,
                "Domains",
                [
                    {"name": "Name", "type": "title"},
                    {"name": "Description", "type": "rich_text"},
                    {"name": "Color", "type": "select", "select": {"options": [
                        {"name": "Red", "color": "red"},
                        {"name": "Orange", "color": "orange"},
                        {"name": "Yellow", "color": "yellow"},
                        {"name": "Green", "color": "green"},
                        {"name": "Blue", "color": "blue"},
                        {"name": "Purple", "color": "purple"},
                        {"name": "Gray", "color": "gray"},
                    ]}},
                ]
            )
            domain_id = domain_db.id
            print(f"[DEBUG] Domains database created: {domain_id}")
        domain_db = type('obj', (object,), {'id': domain_id})  # Mock object with id attribute

        # Step 2: Create Tags database (or reuse existing)
        existing_tags_id = await find_existing_database("Tags")
        if existing_tags_id:
            print(f"[DEBUG] Reusing existing Tags database: {existing_tags_id}")
            tag_id = existing_tags_id
        else:
            print(f"[DEBUG] Creating new Tags database")
            tag_db = await self._create_database(
                parent_page_id,
                "Tags",
                [
                    {"name": "Name", "type": "title"},
                    {"name": "Color", "type": "select", "select": {"options": [
                        {"name": "Red", "color": "red"},
                        {"name": "Orange", "color": "orange"},
                        {"name": "Yellow", "color": "yellow"},
                        {"name": "Green", "color": "green"},
                        {"name": "Blue", "color": "blue"},
                        {"name": "Purple", "color": "purple"},
                        {"name": "Gray", "color": "gray"},
                    ]}},
                    {"name": "Category", "type": "select", "select": {"options": [
                        {"name": "Context"},
                        {"name": "Energy"},
                        {"name": "Location"},
                        {"name": "Time"},
                        {"name": "Custom"},
                    ]}},
                    {"name": "Description", "type": "rich_text"},
                ]
            )
            tag_id = tag_db.id
            print(f"[DEBUG] Tags database created: {tag_id}")
        tag_db = type('obj', (object,), {'id': tag_id})  # Mock object with id attribute

        # Step 3: Create Projects database (or reuse existing)
        existing_projects_id = await find_existing_database("Projects")
        if existing_projects_id:
            print(f"[DEBUG] Reusing existing Projects database: {existing_projects_id}")
            project_id = existing_projects_id
        else:
            print(f"[DEBUG] Creating new Projects database")
            project_db = await self._create_database(
                parent_page_id,
                "Projects",
                [
                    {"name": "Name", "type": "title"},
                    {"name": "Status", "type": "select", "select": {"options": [
                        {"name": "Active", "color": "green"},
                        {"name": "On Hold", "color": "orange"},
                        {"name": "Completed", "color": "blue"},
                        {"name": "Archived", "color": "gray"},
                    ]}},
                    {"name": "Domain", "type": "relation", "relation": {
                        "data_source_id": domain_db.id,
                    }},
                    {"name": "Deadline", "type": "date"},
                    {"name": "Priority", "type": "select", "select": {"options": [
                        {"name": "Critical", "color": "red"},
                        {"name": "High", "color": "orange"},
                        {"name": "Medium", "color": "yellow"},
                        {"name": "Low", "color": "blue"},
                    ]}},
                    {"name": "Progress", "type": "number", "number": {"format": "percent"}},
                    {"name": "Goal", "type": "rich_text"},
                    {"name": "Notes", "type": "rich_text"},
                ]
            )
            project_id = project_db.id
            print(f"[DEBUG] Projects database created: {project_id}")
        project_db = type('obj', (object,), {'id': project_id})  # Mock object with id attribute

        # Step 4: Create Tasks database (or reuse existing)
        existing_tasks_id = await find_existing_database("Tasks")
        if existing_tasks_id:
            print(f"[DEBUG] Reusing existing Tasks database: {existing_tasks_id}")
            task_id = existing_tasks_id
        else:
            print(f"[DEBUG] Creating new Tasks database")
            task_db = await self._create_database(
                parent_page_id,
                "Tasks",
                [
                    {"name": "Title", "type": "title"},
                    {"name": "Status", "type": "select", "select": {"options": [
                        {"name": "Todo", "color": "gray"},
                        {"name": "In Progress", "color": "blue"},
                        {"name": "Done", "color": "green"},
                        {"name": "Cancelled", "color": "red"},
                        {"name": "Archived", "color": "gray"},
                    ]}},
                    {"name": "Priority", "type": "select", "select": {"options": [
                        {"name": "Critical", "color": "red"},
                        {"name": "High", "color": "orange"},
                        {"name": "Medium", "color": "yellow"},
                        {"name": "Low", "color": "blue"},
                    ]}},
                    {"name": "Due Date", "type": "date"},
                    {"name": "Domain", "type": "relation", "relation": {
                        "data_source_id": domain_db.id,
                    }},
                    {"name": "Project", "type": "relation", "relation": {
                        "data_source_id": project_db.id,
                    }},
                    # Note: Parent Task (self-referential) and Subtasks (rollup) properties
                    # need to be added after database creation via a separate API update
                    # because task_db.id doesn't exist yet at this point
                    # TODO: Implement update operation to add these properties
                    {"name": "Tags", "type": "relation", "relation": {
                        "data_source_id": tag_db.id,
                        "dual_property": {
                            "synced_property_name": "Tasks"
                        }
                    }},
                    {"name": "Estimated Time", "type": "number"},
                    {"name": "Energy Required", "type": "select", "select": {"options": [
                        {"name": "High", "color": "red"},
                        {"name": "Medium", "color": "yellow"},
                        {"name": "Low", "color": "blue"},
                    ]}},
                    {"name": "Context", "type": "rich_text"},
                    {"name": "Created Date", "type": "date"},
                    {"name": "Completed Date", "type": "date"},
                    {"name": "Archived Date", "type": "date"},
                ]
            )
            task_id = task_db.id
            print(f"[DEBUG] Tasks database created: {task_id}")
        task_db = type('obj', (object,), {'id': task_id})  # Mock object with id attribute

        # Step 5: Create Routines database (or reuse existing)
        existing_routines_id = await find_existing_database("Routines")
        if existing_routines_id:
            print(f"[DEBUG] Reusing existing Routines database: {existing_routines_id}")
            routine_id = existing_routines_id
        else:
            print(f"[DEBUG] Creating new Routines database")
            routine_db = await self._create_database(
                parent_page_id,
                "Routines",
                [
                    {"name": "Name", "type": "title"},
                    {"name": "Frequency", "type": "select", "select": {"options": [
                        {"name": "Daily", "color": "blue"},
                        {"name": "Weekly", "color": "green"},
                        {"name": "Weekdays", "color": "yellow"},
                        {"name": "Weekends", "color": "purple"},
                    ]}},
                    {"name": "Domain", "type": "relation", "relation": {
                        "data_source_id": domain_db.id,
                    }},
                    {"name": "Best Time", "type": "rich_text"},
                    {"name": "Estimated Duration", "type": "number"},
                    {"name": "Streak", "type": "number"},
                    {"name": "Last Completed", "type": "date"},
                    {"name": "Total Completions", "type": "number"},
                ]
            )
            routine_id = routine_db.id
            print(f"[DEBUG] Routines database created: {routine_id}")
        routine_db = type('obj', (object,), {'id': routine_id})  # Mock object with id attribute

        # Step 6: Create Agenda database (or reuse existing)
        existing_agenda_id = await find_existing_database("Agenda")
        if existing_agenda_id:
            print(f"[DEBUG] Reusing existing Agenda database: {existing_agenda_id}")
            agenda_id = existing_agenda_id
        else:
            print(f"[DEBUG] Creating new Agenda database")
            agenda_db = await self._create_database(
                parent_page_id,
                "Agenda",
                [
                    {"name": "Name", "type": "title"},
                    {"name": "Date & Time", "type": "date"},
                    {"name": "Duration", "type": "number"},
                    {"name": "Type", "type": "select", "select": {"options": [
                        {"name": "Event", "color": "blue"},
                        {"name": "Time Block", "color": "green"},
                        {"name": "Reminder", "color": "yellow"},
                    ]}},
                    {"name": "Linked Task", "type": "relation", "relation": {
                        "data_source_id": task_db.id,
                    }},
                    {"name": "Linked Project", "type": "relation", "relation": {
                        "data_source_id": project_db.id,
                    }},
                    {"name": "Location", "type": "rich_text"},
                    {"name": "Notes", "type": "rich_text"},
                ]
            )
            agenda_id = agenda_db.id
            print(f"[DEBUG] Agenda database created: {agenda_id}")
        agenda_db = type('obj', (object,), {'id': agenda_id})  # Mock object with id attribute

        # Store database IDs
        self._database_ids = {
            "domains": domain_db.id,
            "tags": tag_db.id,
            "projects": project_db.id,
            "tasks": task_db.id,
            "routines": routine_db.id,
            "agenda": agenda_db.id,
        }

        # Save configuration
        self.save_database_ids()

        return self._database_ids

    async def _create_database(
        self,
        parent_page_id: str,
        title: str,
        properties: list[dict],
    ):
        """Create a Notion database with given properties.

        Args:
            parent_page_id: Parent page ID
            title: Database title
            properties: List of property definitions

        Returns:
            Created Database object
        """

        # Format properties for API
        schema_properties = []
        for prop in properties:
            prop_schema = {"name": prop["name"], "type": prop["type"]}

            if prop["type"] == "title":
                prop_schema["title"] = {}
            elif prop["type"] == "rich_text":
                prop_schema["rich_text"] = {}
            elif prop["type"] == "number":
                prop_schema["number"] = {"format": prop.get("format", "number")}
            elif prop["type"] == "select":
                prop_schema["select"] = prop["select"]
            elif prop["type"] == "relation":
                prop_schema["relation"] = prop["relation"]
            elif prop["type"] == "rollup":
                prop_schema["rollup"] = prop["rollup"]
            # For date, checkbox, etc.: no additional fields needed

            schema_properties.append(prop_schema)

        # Create database via API using the DatabaseCollection
        from better_notion._api.collections import DatabaseCollection

        # Create a DatabaseCollection instance
        db_collection = DatabaseCollection(self.client._api)

        # Log the request payload for debugging
        import json
        request_payload = {
            "parent": {"type": "page_id", "page_id": parent_page_id},
            "title": title,
            "properties": schema_properties,
        }
        print(f"[DEBUG] Creating database '{title}' with schema:", file=__import__('sys').stderr)
        print(json.dumps(request_payload, indent=2), file=__import__('sys').stderr)

        try:
            response = await db_collection.create(
                parent=request_payload["parent"],
                title=title,
                properties=schema_properties,
            )
            print(f"[DEBUG] Database '{title}' created successfully: {response.get('id')}", file=__import__('sys').stderr)
        except Exception as e:
            print(f"[ERROR] Failed to create database '{title}': {e}", file=__import__('sys').stderr)
            if hasattr(e, 'response'):
                print(f"[ERROR] API Response: {e.response.text if hasattr(e.response, 'text') else str(e)}", file=__import__('sys').stderr)
            raise

        from better_notion._sdk.models.database import Database
        return Database(data=response, client=self.client)

    def save_database_ids(self) -> None:
        """Save database IDs and workspace configuration to disk."""
        config = {
            "workspace_id": self._workspace_id,
            "workspace_name": self._workspace_name,
            "parent_page_id": self._parent_page_id,
            "database_ids": self._database_ids,
            "initialized_at": datetime.now().isoformat(),
            "version": "1.0.0",
        }

        config_path = Path.home() / ".notion" / "personal.json"
        config_path.parent.mkdir(parents=True, exist_ok=True)

        with open(config_path, "w", encoding="utf-8") as f:
            json.dump(config, f, indent=2)

    @staticmethod
    def load_parent_page() -> str | None:
        """Load parent page ID from saved config.

        Returns:
            Parent page ID if found, None otherwise
        """
        from better_notion.utils.personal.metadata import PersonalWorkspaceMetadata, CONFIG_PATH

        if CONFIG_PATH.exists():
            with open(CONFIG_PATH, encoding="utf-8") as f:
                config = json.load(f)
                return config.get("parent_page_id")
        return None



"""Workflow entity models for the agents SDK plugin.

This module provides SDK model classes for workflow entities:
- Organization
- Project
- Version
- Task

These models inherit from BaseEntity and provide autonomous CRUD operations
with caching support through the plugin system.
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, AsyncIterator

from better_notion._sdk.base.entity import BaseEntity

if TYPE_CHECKING:
    from better_notion._sdk.client import NotionClient
    from better_notion._sdk.models.block import Block
    from better_notion._sdk.models.database import Database
    from better_notion._sdk.models.page import Page


class DatabasePageEntityMixin:
    """Mixin providing BaseEntity abstract method implementations for database pages.

    This mixin provides parent() and children() implementations for entities
    that are pages in databases (like Organization, Project, etc.).
    """

    async def parent(self) -> "Database | Page | None":
        """Get parent object (database for entity pages).

        Returns:
            Parent Database or Page or None
        """
        from better_notion._sdk.models.database import Database

        # Get parent from data
        parent_data = self._data.get("parent")
        if not parent_data:
            return None

        # Check cache
        cached = self._cache_get("parent")
        if cached:
            return cached

        # Database parent
        if parent_data.get("type") == "database_id":
            db_id = parent_data.get("database_id")
            parent = await Database.get(db_id, client=self._client)
            self._cache_set("parent", parent)
            return parent

        # Page parent
        if parent_data.get("type") == "page_id":
            from better_notion._sdk.models.page import Page
            page_id = parent_data.get("page_id")
            parent = await Page.get(page_id, client=self._client)
            self._cache_set("parent", parent)
            return parent

        return None

    async def children(self) -> AsyncIterator["Block"]:
        """Iterate over child blocks.

        Yields:
            Block objects that are direct children
        """
        from better_notion._api.utils.pagination import AsyncPaginatedIterator
        from better_notion._sdk.models.block import Block

        async def _get_blocks(offset: str | None = None) -> dict[str, Any]:
            return await self._client._api._request(
                "GET",
                f"/blocks/{self.id}/children",
                params={"start_cursor": offset} if offset else None
            )

        iterator = AsyncPaginatedIterator(_get_blocks, lambda data: data.get("results", []))

        async for block_data in iterator:
            yield Block(self._client, block_data)


class Organization(DatabasePageEntityMixin, BaseEntity):
    """
    Organization entity representing a company or team.

    An organization contains multiple projects and serves as the top-level
    grouping in the workflow hierarchy.

    Attributes:
        id: Organization page ID
        name: Organization name
        slug: URL-safe identifier
        description: Organization purpose
        repository_url: Code repository URL
        status: Organization status (Active, Archived, On Hold)

    Example:
        >>> org = await Organization.get("org_id", client=client)
        >>> print(org.name)
        >>> projects = await org.projects()
    """

    def __init__(self, client: "NotionClient", data: dict[str, Any]) -> None:
        """Initialize organization with client and API data.

        Args:
            client: NotionClient instance
            data: Raw API response data
        """
        super().__init__(client, data)
        self._organization_cache = client.plugin_cache("organizations")

    # ===== PROPERTIES =====

    @property
    def name(self) -> str:
        """Get organization name from title property.

        Returns:
            Organization name as string
        """
        title_prop = self._data["properties"].get("Name") or self._data["properties"].get("name")
        if title_prop and title_prop.get("type") == "title":
            title_data = title_prop.get("title", [])
            if title_data:
                return title_data[0].get("plain_text", "")
        return ""

    @property
    def slug(self) -> str:
        """Get organization slug (URL-safe identifier).

        Returns:
            Slug string or empty string if not set
        """
        slug_prop = self._data["properties"].get("Slug") or self._data["properties"].get("slug")
        if slug_prop and slug_prop.get("type") == "rich_text":
            text_data = slug_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def description(self) -> str:
        """Get organization description.

        Returns:
            Description string
        """
        desc_prop = self._data["properties"].get("Description") or self._data["properties"].get("description")
        if desc_prop and desc_prop.get("type") == "rich_text":
            text_data = desc_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def repository_url(self) -> str | None:
        """Get repository URL.

        Returns:
            Repository URL string or None
        """
        repo_prop = self._data["properties"].get("Repository URL") or self._data["properties"].get("repository_url")
        if repo_prop and repo_prop.get("type") == "url":
            return repo_prop.get("url")
        return None

    @property
    def status(self) -> str:
        """Get organization status.

        Returns:
            Status string (Active, Archived, On Hold)
        """
        status_prop = self._data["properties"].get("Status") or self._data["properties"].get("status")
        if status_prop and status_prop.get("type") == "select":
            select_data = status_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    # ===== AUTONOMOUS METHODS =====

    @classmethod
    async def get(cls, org_id: str, *, client: "NotionClient") -> "Organization":
        """
        Get an organization by ID.

        Args:
            org_id: Organization page ID
            client: NotionClient instance

        Returns:
            Organization instance

        Raises:
            Exception: If API call fails

        Example:
            >>> org = await Organization.get("org_123", client=client)
        """
        # Check plugin cache
        cache = client.plugin_cache("organizations")
        if cache and org_id in cache:
            return cache[org_id]

        # Fetch from API
        data = await client._api.pages.get(page_id=org_id)
        org = cls(client, data)

        # Cache it
        if cache:
            cache[org_id] = org

        return org

    @classmethod
    async def create(
        cls,
        *,
        client: "NotionClient",
        database_id: str,
        name: str,
        slug: str | None = None,
        description: str | None = None,
        repository_url: str | None = None,
        status: str = "Active",
    ) -> "Organization":
        """
        Create a new organization.

        Args:
            client: NotionClient instance
            database_id: Organizations database ID
            name: Organization name
            slug: URL-safe identifier (defaults to name if not provided)
            description: Organization description
            repository_url: Code repository URL
            status: Organization status (default: Active)

        Returns:
            Created Organization instance

        Raises:
            Exception: If API call fails

        Example:
            >>> org = await Organization.create(
            ...     client=client,
            ...     database_id="db_123",
            ...     name="My Organization",
            ...     slug="my-org",
            ...     description="A great organization"
            ... )
        """
        from better_notion._api.properties import Title, RichText, URL, Select

        # Build properties
        properties: dict[str, Any] = {
            "Name": Title(content=name),
        }

        if slug:
            properties["Slug"] = RichText(name="Slug", content=slug)
        if description:
            properties["Description"] = RichText(name="Description", content=description)
        if repository_url:
            properties["Repository URL"] = URL(name="Repository URL", url=repository_url)
        if status:
            properties["Status"] = Select(name="Status", value=status)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        # Create page
        data = await client._api.pages.create(
            parent={"database_id": database_id},
            properties=serialized_properties,
        )

        org = cls(client, data)

        # Cache it
        cache = client.plugin_cache("organizations")
        if cache:
            cache[org.id] = org

        return org

    async def update(
        self,
        *,
        name: str | None = None,
        slug: str | None = None,
        description: str | None = None,
        repository_url: str | None = None,
        status: str | None = None,
    ) -> "Organization":
        """
        Update organization properties.

        Args:
            name: New name
            slug: New slug
            description: New description
            repository_url: New repository URL
            status: New status

        Returns:
            Updated Organization instance

        Example:
            >>> org = await org.update(status="Archived")
        """
        from better_notion._api.properties import Title, RichText, URL, Select

        # Build properties to update
        properties: dict[str, Any] = {}

        if name is not None:
            properties["Name"] = Title(content=name)
        if slug is not None:
            properties["Slug"] = RichText(name="Slug", content=slug)
        if description is not None:
            properties["Description"] = RichText(name="Description", content=description)
        if repository_url is not None:
            properties["Repository URL"] = URL(name="Repository URL", url=repository_url)
        if status is not None:
            properties["Status"] = Select(name="Status", value=status)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        # Update page
        data = await self._client._api.pages.update(
            page_id=self.id,
            properties=serialized_properties,
        )

        # Update instance
        self._data = data

        # Invalidate cache
        cache = self._client.plugin_cache("organizations")
        if cache and self.id in cache:
            del cache[self.id]

        return self

    async def delete(self) -> None:
        """
        Delete the organization.

        Example:
            >>> await org.delete()
        """
        await self._client._api.pages.delete(page_id=self.id)

        # Invalidate cache
        cache = self._client.plugin_cache("organizations")
        if cache and self.id in cache:
            del cache[self.id]

    async def projects(self) -> list["Project"]:
        """
        Get all projects in this organization.

        Returns:
            List of Project instances

        Example:
            >>> projects = await org.projects()
        """
        # Get database ID from workspace config
        import json
        from pathlib import Path

        config_path = Path.home() / ".notion" / "workspace.json"
        if not config_path.exists():
            return []

        config = json.loads(config_path.read_text())
        projects_db_id = config.get("Projects")

        if not projects_db_id:
            return []

        # Query projects with relation filter
        from better_notion._api.properties import Relation

        response = await self._client._api.databases.query(
            database_id=projects_db_id,
            filter={
                "property": "Organization",
                "relation": {"contains": self.id},
            },
        )

        from better_notion.plugins.official.agents_sdk.models import Project
        return [Project(self._client, page_data) for page_data in response.get("results", [])]


class Project(DatabasePageEntityMixin, BaseEntity):
    """
    Project entity representing a software project.

    A project belongs to an organization and contains multiple versions.

    Attributes:
        id: Project page ID
        name: Project name
        slug: URL-safe identifier
        description: Project description
        repository: Git repository URL
        status: Project status (Active, Archived, Planning, Completed)
        tech_stack: List of technologies used
        role: Project role (Developer, PM, QA, etc.)
        organization: Parent organization

    Example:
        >>> project = await Project.get("project_id", client=client)
        >>> print(project.name)
        >>> versions = await project.versions()
    """

    def __init__(self, client: "NotionClient", data: dict[str, Any]) -> None:
        """Initialize project with client and API data.

        Args:
            client: NotionClient instance
            data: Raw API response data
        """
        super().__init__(client, data)
        self._project_cache = client.plugin_cache("projects")

    # ===== PROPERTIES =====

    @property
    def name(self) -> str:
        """Get project name from title property."""
        title_prop = self._data["properties"].get("Name") or self._data["properties"].get("name")
        if title_prop and title_prop.get("type") == "title":
            title_data = title_prop.get("title", [])
            if title_data:
                return title_data[0].get("plain_text", "")
        return ""

    @property
    def slug(self) -> str:
        """Get project slug."""
        slug_prop = self._data["properties"].get("Slug") or self._data["properties"].get("slug")
        if slug_prop and slug_prop.get("type") == "rich_text":
            text_data = slug_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def description(self) -> str:
        """Get project description."""
        desc_prop = self._data["properties"].get("Description") or self._data["properties"].get("description")
        if desc_prop and desc_prop.get("type") == "rich_text":
            text_data = desc_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def repository(self) -> str | None:
        """Get repository URL."""
        repo_prop = self._data["properties"].get("Repository") or self._data["properties"].get("repository")
        if repo_prop and repo_prop.get("type") == "url":
            return repo_prop.get("url")
        return None

    @property
    def status(self) -> str:
        """Get project status."""
        status_prop = self._data["properties"].get("Status") or self._data["properties"].get("status")
        if status_prop and status_prop.get("type") == "select":
            select_data = status_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def tech_stack(self) -> list[str]:
        """Get tech stack as list of strings."""
        tech_prop = self._data["properties"].get("Tech Stack") or self._data["properties"].get("tech_stack")
        if tech_prop and tech_prop.get("type") == "multi_select":
            multi_data = tech_prop.get("multi_select", [])
            return [item.get("name", "") for item in multi_data]
        return []

    @property
    def role(self) -> str:
        """Get project role."""
        role_prop = self._data["properties"].get("Role") or self._data["properties"].get("role")
        if role_prop and role_prop.get("type") == "select":
            select_data = role_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def organization_id(self) -> str | None:
        """Get parent organization ID."""
        org_prop = self._data["properties"].get("Organization") or self._data["properties"].get("organization")
        if org_prop and org_prop.get("type") == "relation":
            relations = org_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    # ===== AUTONOMOUS METHODS =====

    @classmethod
    async def get(cls, project_id: str, *, client: "NotionClient") -> "Project":
        """Get a project by ID."""
        # Check plugin cache
        cache = client.plugin_cache("projects")
        if cache and project_id in cache:
            return cache[project_id]

        # Fetch from API
        data = await client._api.pages.get(page_id=project_id)
        project = cls(client, data)

        # Cache it
        if cache:
            cache[project_id] = project

        return project

    @classmethod
    async def create(
        cls,
        *,
        client: "NotionClient",
        database_id: str,
        name: str,
        organization_id: str,
        slug: str | None = None,
        description: str | None = None,
        repository: str | None = None,
        status: str = "Active",
        tech_stack: list[str] | None = None,
        role: str = "Developer",
    ) -> "Project":
        """Create a new project."""
        from better_notion._api.properties import Title, RichText, URL, Select, MultiSelect, Relation

        # Build properties
        properties: dict[str, Any] = {
            "Name": Title(content=name),
            "Organization": Relation([organization_id]),
            "Role": Select(name="Role", value=role),
        }

        if slug:
            properties["Slug"] = RichText(name="Slug", content=slug)
        if description:
            properties["Description"] = RichText(name="Description", content=description)
        if repository:
            properties["Repository"] = URL(name="Repository", url=repository)
        if status:
            properties["Status"] = Select(name="Status", value=status)
        if tech_stack:
            properties["Tech Stack"] = MultiSelect(name="Tech Stack", values=tech_stack)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        # Create page
        data = await client._api.pages.create(
            parent={"database_id": database_id},
            properties=serialized_properties,
        )

        project = cls(client, data)

        # Cache it
        cache = client.plugin_cache("projects")
        if cache:
            cache[project.id] = project

        return project

    async def update(
        self,
        *,
        name: str | None = None,
        slug: str | None = None,
        description: str | None = None,
        repository: str | None = None,
        status: str | None = None,
        tech_stack: list[str] | None = None,
        role: str | None = None,
    ) -> "Project":
        """Update project properties."""
        from better_notion._api.properties import Title, RichText, URL, Select, MultiSelect

        # Build properties to update
        properties: dict[str, Any] = {}

        if name is not None:
            properties["Name"] = Title(content=name)
        if slug is not None:
            properties["Slug"] = RichText(name="Slug", content=slug)
        if description is not None:
            properties["Description"] = RichText(name="Description", content=description)
        if repository is not None:
            properties["Repository"] = URL(name="Repository", url=repository)
        if status is not None:
            properties["Status"] = Select(name="Status", value=status)
        if tech_stack is not None:
            properties["Tech Stack"] = MultiSelect(name="Tech Stack", values=tech_stack)
        if role is not None:
            properties["Role"] = Select(name="Role", value=role)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        # Update page
        data = await self._client._api.pages.update(
            page_id=self.id,
            properties=serialized_properties,
        )

        # Update instance
        self._data = data

        # Invalidate cache
        cache = self._client.plugin_cache("projects")
        if cache and self.id in cache:
            del cache[self.id]

        return self

    async def delete(self) -> None:
        """Delete the project."""
        await self._client._api.pages.delete(page_id=self.id)

        # Invalidate cache
        cache = self._client.plugin_cache("projects")
        if cache and self.id in cache:
            del cache[self.id]

    async def organization(self) -> Organization | None:
        """Get parent organization."""
        if self.organization_id:
            return await Organization.get(self.organization_id, client=self._client)
        return None

    async def versions(self) -> list["Version"]:
        """Get all versions in this project."""
        # Get database ID from workspace config
        import json
        from pathlib import Path

        config_path = Path.home() / ".notion" / "workspace.json"
        if not config_path.exists():
            return []

        config = json.loads(config_path.read_text())
        versions_db_id = config.get("Versions")

        if not versions_db_id:
            return []

        # Query versions with relation filter
        response = await self._client._api.databases.query(
            database_id=versions_db_id,
            filter={
                "property": "Project",
                "relation": {"contains": self.id},
            },
        )

        return [Version(self._client, page_data) for page_data in response.get("results", [])]


class Version(DatabasePageEntityMixin, BaseEntity):
    """
    Version entity representing a project version/release.

    A version belongs to a project and contains multiple tasks.

    Attributes:
        id: Version page ID
        name: Version name (e.g., "v1.0.0")
        project: Parent project
        status: Version status (Planning, Alpha, Beta, RC, In Progress, Released)
        type: Version type (Major, Minor, Patch, Hotfix)
        branch_name: Git branch name
        progress: Progress percentage (0-100)

    Example:
        >>> version = await Version.get("version_id", client=client)
        >>> print(version.name)
        >>> tasks = await version.tasks()
    """

    def __init__(self, client: "NotionClient", data: dict[str, Any]) -> None:
        """Initialize version with client and API data."""
        super().__init__(client, data)
        self._version_cache = client.plugin_cache("versions")

    # ===== PROPERTIES =====

    @property
    def name(self) -> str:
        """Get version name from title property."""
        title_prop = self._data["properties"].get("Version") or self._data["properties"].get("version")
        if title_prop and title_prop.get("type") == "title":
            title_data = title_prop.get("title", [])
            if title_data:
                return title_data[0].get("plain_text", "")
        return ""

    @property
    def status(self) -> str:
        """Get version status."""
        status_prop = self._data["properties"].get("Status") or self._data["properties"].get("status")
        if status_prop and status_prop.get("type") == "select":
            select_data = status_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def version_type(self) -> str:
        """Get version type."""
        type_prop = self._data["properties"].get("Type") or self._data["properties"].get("type")
        if type_prop and type_prop.get("type") == "select":
            select_data = type_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def branch_name(self) -> str:
        """Get git branch name."""
        branch_prop = self._data["properties"].get("Branch Name") or self._data["properties"].get("branch_name")
        if branch_prop and branch_prop.get("type") == "rich_text":
            text_data = branch_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def progress(self) -> int:
        """Get progress percentage."""
        progress_prop = self._data["properties"].get("Progress") or self._data["properties"].get("progress")
        if progress_prop and progress_prop.get("type") == "number":
            return progress_prop.get("number", 0) or 0
        return 0

    @property
    def project_id(self) -> str | None:
        """Get parent project ID."""
        project_prop = self._data["properties"].get("Project") or self._data["properties"].get("project")
        if project_prop and project_prop.get("type") == "relation":
            relations = project_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    # ===== AUTONOMOUS METHODS =====

    @classmethod
    async def get(cls, version_id: str, *, client: "NotionClient") -> "Version":
        """Get a version by ID."""
        # Check plugin cache
        cache = client.plugin_cache("versions")
        if cache and version_id in cache:
            return cache[version_id]

        # Fetch from API
        data = await client._api.pages.get(page_id=version_id)
        version = cls(client, data)

        # Cache it
        if cache:
            cache[version_id] = version

        return version

    @classmethod
    async def create(
        cls,
        *,
        client: "NotionClient",
        database_id: str,
        name: str,
        project_id: str,
        status: str = "Planning",
        version_type: str = "Minor",
        branch_name: str | None = None,
        progress: int = 0,
    ) -> "Version":
        """Create a new version."""
        from better_notion._api.properties import Title, Select, RichText, Number, Relation

        # Build properties
        properties: dict[str, Any] = {
            "Version": Title(content=name),
            "Project": Relation([project_id]),
            "Status": Select(name="Status", value=status),
            "Type": Select(name="Type", value=version_type),
            "Progress": Number(name="Progress", value=progress),
        }

        if branch_name:
            properties["Branch Name"] = RichText(name="Branch Name", content=branch_name)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        # Create page
        data = await client._api.pages.create(
            parent={"database_id": database_id},
            properties=serialized_properties,
        )

        version = cls(client, data)

        # Cache it
        cache = client.plugin_cache("versions")
        if cache:
            cache[version.id] = version

        return version

    async def update(
        self,
        *,
        name: str | None = None,
        status: str | None = None,
        version_type: str | None = None,
        branch_name: str | None = None,
        progress: int | None = None,
    ) -> "Version":
        """Update version properties."""
        from better_notion._api.properties import Title, Select, RichText, Number

        # Build properties to update
        properties: dict[str, Any] = {}

        if name is not None:
            properties["Version"] = Title(content=name)
        if status is not None:
            properties["Status"] = Select(name="Status", value=status)
        if version_type is not None:
            properties["Type"] = Select(name="Type", value=version_type)
        if branch_name is not None:
            properties["Branch Name"] = RichText(name="Branch Name", content=branch_name)
        if progress is not None:
            properties["Progress"] = Number(name="Progress", value=progress)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        # Update page
        data = await self._client._api.pages.update(
            page_id=self.id,
            properties=serialized_properties,
        )

        # Update instance
        self._data = data

        # Invalidate cache
        cache = self._client.plugin_cache("versions")
        if cache and self.id in cache:
            del cache[self.id]

        return self

    async def delete(self) -> None:
        """Delete the version."""
        await self._client._api.pages.delete(page_id=self.id)

        # Invalidate cache
        cache = self._client.plugin_cache("versions")
        if cache and self.id in cache:
            del cache[self.id]

    async def project(self) -> Project | None:
        """Get parent project."""
        if self.project_id:
            return await Project.get(self.project_id, client=self._client)
        return None

    async def tasks(self) -> list["Task"]:
        """Get all tasks in this version."""
        # Get database ID from workspace config
        import json
        from pathlib import Path

        config_path = Path.home() / ".notion" / "workspace.json"
        if not config_path.exists():
            return []

        config = json.loads(config_path.read_text())
        tasks_db_id = config.get("Tasks")

        if not tasks_db_id:
            return []

        # Query tasks with relation filter
        response = await self._client._api.databases.query(
            database_id=tasks_db_id,
            filter={
                "property": "Version",
                "relation": {"contains": self.id},
            },
        )

        return [Task(self._client, page_data) for page_data in response.get("results", [])]


class Task(DatabasePageEntityMixin, BaseEntity):
    """
    Task entity representing a work task.

    A task belongs to a version and can have dependencies on other tasks.

    Attributes:
        id: Task page ID
        title: Task title
        version: Parent version
        status: Task status (Backlog, Claimed, In Progress, In Review, Completed)
        type: Task type (New Feature, Refactor, Documentation, Test, Bug Fix)
        priority: Task priority (Critical, High, Medium, Low)
        dependencies: List of task IDs this task depends on
        estimated_hours: Estimated hours to complete
        actual_hours: Actual hours spent

    Example:
        >>> task = await Task.get("task_id", client=client)
        >>> await task.claim()
        >>> await task.start()
        >>> await task.complete()
    """

    def __init__(self, client: "NotionClient", data: dict[str, Any]) -> None:
        """Initialize task with client and API data."""
        super().__init__(client, data)
        self._task_cache = client.plugin_cache("tasks")

    # ===== PROPERTIES =====

    @property
    def title(self) -> str:
        """Get task title from title property."""
        title_prop = self._data["properties"].get("Title") or self._data["properties"].get("title")
        if title_prop and title_prop.get("type") == "title":
            title_data = title_prop.get("title", [])
            if title_data:
                return title_data[0].get("plain_text", "")
        return ""

    @property
    def status(self) -> str:
        """Get task status."""
        status_prop = self._data["properties"].get("Status") or self._data["properties"].get("status")
        if status_prop and status_prop.get("type") == "select":
            select_data = status_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def task_type(self) -> str:
        """Get task type."""
        type_prop = self._data["properties"].get("Type") or self._data["properties"].get("type")
        if type_prop and type_prop.get("type") == "select":
            select_data = type_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def priority(self) -> str:
        """Get task priority."""
        priority_prop = self._data["properties"].get("Priority") or self._data["properties"].get("priority")
        if priority_prop and priority_prop.get("type") == "select":
            select_data = priority_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def version_id(self) -> str | None:
        """Get parent version ID."""
        version_prop = self._data["properties"].get("Version") or self._data["properties"].get("version")
        if version_prop and version_prop.get("type") == "relation":
            relations = version_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    @property
    def dependency_ids(self) -> list[str]:
        """Get list of task IDs this task depends on."""
        dep_prop = self._data["properties"].get("Dependencies") or self._data["properties"].get("dependencies")
        if dep_prop and dep_prop.get("type") == "relation":
            relations = dep_prop.get("relation", [])
            return [r.get("id", "") for r in relations if r.get("id")]
        return []

    @property
    def estimated_hours(self) -> int | None:
        """Get estimated hours."""
        hours_prop = self._data["properties"].get("Estimated Hours") or self._data["properties"].get("estimated_hours")
        if hours_prop and hours_prop.get("type") == "number":
            return hours_prop.get("number")
        return None

    @property
    def actual_hours(self) -> int | None:
        """Get actual hours spent."""
        hours_prop = self._data["properties"].get("Actual Hours") or self._data["properties"].get("actual_hours")
        if hours_prop and hours_prop.get("type") == "number":
            return hours_prop.get("number")
        return None

    @property
    def assignee(self) -> str | None:
        """Get the assignee of this task."""
        assignee_prop = self._data["properties"].get("Assignee") or self._data["properties"].get("assignee")
        if assignee_prop and assignee_prop.get("type") == "select":
            select_data = assignee_prop.get("select")
            if select_data:
                return select_data.get("name")
        return None

    @property
    def related_work_issue_id(self) -> str | None:
        """Get related work issue ID (blocking this task or caused by this task)."""
        issue_prop = self._data["properties"].get("Related Work Issue") or self._data["properties"].get("related_work_issue")
        if issue_prop and issue_prop.get("type") == "relation":
            relations = issue_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    # ===== AUTONOMOUS METHODS =====

    @classmethod
    async def get(cls, task_id: str, *, client: "NotionClient") -> "Task":
        """Get a task by ID."""
        # Check plugin cache
        cache = client.plugin_cache("tasks")
        if cache and task_id in cache:
            return cache[task_id]

        # Fetch from API
        data = await client._api.pages.get(page_id=task_id)
        task = cls(client, data)

        # Cache it
        if cache:
            cache[task_id] = task

        return task

    @classmethod
    async def create(
        cls,
        *,
        client: "NotionClient",
        database_id: str,
        title: str,
        version_id: str,
        status: str = "Backlog",
        task_type: str = "New Feature",
        priority: str = "Medium",
        dependency_ids: list[str] | None = None,
        estimated_hours: int | None = None,
    ) -> "Task":
        """Create a new task."""
        from better_notion._api.properties import Title, Select, Number, Relation

        # Build properties
        properties: dict[str, Any] = {
            "Title": Title(content=title),
            "Version": Relation([version_id]),
            "Status": Select(name="Status", value=status),
            "Type": Select(name="Type", value=task_type),
            "Priority": Select(name="Priority", value=priority),
        }

        if dependency_ids:
            properties["Dependencies"] = Relation(dependency_ids)
        if estimated_hours is not None:
            properties["Estimated Hours"] = Number(name="Estimated Hours", value=estimated_hours)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        # Create page
        data = await client._api.pages.create(
            parent={"database_id": database_id},
            properties=serialized_properties,
        )

        task = cls(client, data)

        # Cache it
        cache = client.plugin_cache("tasks")
        if cache:
            cache[task.id] = task

        return task

    async def update(
        self,
        *,
        title: str | None = None,
        status: str | None = None,
        task_type: str | None = None,
        priority: str | None = None,
        dependency_ids: list[str] | None = None,
        estimated_hours: int | None = None,
        actual_hours: int | None = None,
    ) -> "Task":
        """Update task properties."""
        from better_notion._api.properties import Title, Select, Number, Relation

        # Build properties to update
        properties: dict[str, Any] = {}

        if title is not None:
            properties["Title"] = Title(content=title)
        if status is not None:
            properties["Status"] = Select(name="Status", value=status)
        if task_type is not None:
            properties["Type"] = Select(name="Type", value=task_type)
        if priority is not None:
            properties["Priority"] = Select(name="Priority", value=priority)
        if dependency_ids is not None:
            properties["Dependencies"] = Relation(dependency_ids)
        if estimated_hours is not None:
            properties["Estimated Hours"] = Number(name="Estimated Hours", value=estimated_hours)
        if actual_hours is not None:
            properties["Actual Hours"] = Number(name="Actual Hours", value=actual_hours)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        # Update page
        data = await self._client._api.pages.update(
            page_id=self.id,
            properties=serialized_properties,
        )

        # Update instance
        self._data = data

        # Invalidate cache
        cache = self._client.plugin_cache("tasks")
        if cache and self.id in cache:
            del cache[self.id]

        return self

    async def delete(self) -> None:
        """Delete the task."""
        await self._client._api.pages.delete(page_id=self.id)

        # Invalidate cache
        cache = self._client.plugin_cache("tasks")
        if cache and self.id in cache:
            del cache[self.id]

    async def version(self) -> Version | None:
        """Get parent version."""
        if self.version_id:
            return await Version.get(self.version_id, client=self._client)
        return None

    async def dependencies(self) -> list["Task"]:
        """Get all tasks this task depends on."""
        tasks = []
        for dep_id in self.dependency_ids:
            try:
                task = await Task.get(dep_id, client=self._client)
                tasks.append(task)
            except Exception:
                pass
        return tasks

    # ===== WORKFLOW METHODS =====

    async def claim(self) -> "Task":
        """
        Claim this task (transition to Claimed status).

        Returns:
            Updated Task instance

        Example:
            >>> await task.claim()
        """
        return await self.update(status="Claimed")

    async def start(self) -> "Task":
        """
        Start working on this task (transition to In Progress).

        Returns:
            Updated Task instance

        Example:
            >>> await task.start()
        """
        return await self.update(status="In Progress")

    async def complete(self, actual_hours: int | None = None) -> "Task":
        """
        Complete this task (transition to Completed).

        Args:
            actual_hours: Actual hours spent (optional)

        Returns:
            Updated Task instance

        Example:
            >>> await task.complete(actual_hours=3)
        """
        return await self.update(status="Completed", actual_hours=actual_hours)

    async def can_start(self) -> bool:
        """
        Check if this task can start (all dependencies completed).

        Returns:
            True if all dependencies are completed

        Example:
            >>> if await task.can_start():
            ...     await task.start()
        """
        for dep in await self.dependencies():
            if dep.status != "Completed":
                return False
        return True

    async def related_work_issue(self) -> "WorkIssue | None":
        """
        Get the related work issue (blocking this task or caused by this task).

        Returns:
            WorkIssue instance or None

        Example:
            >>> issue = await task.related_work_issue()
        """
        from better_notion.plugins.official.agents_sdk.models import WorkIssue

        issue_id = self.related_work_issue_id
        if not issue_id:
            return None

        try:
            return await WorkIssue.get(issue_id, client=self._client)
        except Exception:
            return None

    async def link_to_work_issue(self, work_issue_id: str) -> None:
        """
        Link this task to a work issue.

        Args:
            work_issue_id: Work issue ID to link to

        Example:
            >>> await task.link_to_work_issue(issue_id)
        """
        from better_notion._api.properties import Relation

        await self._client._api.pages.update(
            page_id=self.id,
            properties={
                "Related Work Issue": Relation([work_issue_id]).to_dict(),
            },
        )

        # Update local data
        self._data["properties"]["Related Work Issue"] = {
            "type": "relation",
            "relation": [{"id": work_issue_id}]
        }

    async def unlink_work_issue(self) -> None:
        """
        Unlink this task from its work issue.

        Example:
            >>> await task.unlink_work_issue()
        """
        from better_notion._api.properties import Relation

        await self._client._api.pages.update(
            page_id=self.id,
            properties={
                "Related Work Issue": Relation([]).to_dict(),
            },
        )

        # Update local data
        self._data["properties"]["Related Work Issue"] = {
            "type": "relation",
            "relation": []
        }

    async def assign_to(self, assignee: str) -> None:
        """
        Assign this task to a person.

        Args:
            assignee: Name of the person to assign to

        Example:
            >>> await task.assign_to("Alice Chen")
        """
        from better_notion._api.properties import Select

        await self._client._api.pages.update(
            page_id=self.id,
            properties={
                "Assignee": Select(name="Assignee", value=assignee).to_dict(),
            },
        )

        # Update local data
        self._data["properties"]["Assignee"] = {
            "type": "select",
            "select": {"name": assignee}
        }

    async def unassign(self) -> None:
        """
        Unassign this task.

        Example:
            >>> await task.unassign()
        """
        from better_notion._api.properties import Select

        await self._client._api.pages.update(
            page_id=self.id,
            properties={
                "Assignee": None,
            },
        )

        # Update local data
        self._data["properties"]["Assignee"] = {
            "type": "select",
            "select": None
        }


class Idea(DatabasePageEntityMixin, BaseEntity):
    """
    Idea entity representing an improvement opportunity discovered during work.

    Ideas enable continuous improvement by capturing innovations and
    problems discovered during development work.

    Attributes:
        id: Idea page ID
        title: Idea title
        category: Idea category (Feature, Improvement, Refactor, etc.)
        status: Idea status (New, Evaluated, Accepted, Rejected, Deferred)
        description: Detailed explanation
        proposed_solution: How to implement
        benefits: Why this is valuable
        effort_estimate: Implementation effort (Small, Medium, Large)
        context: What you were doing when you thought of this
        project_id: Related project ID (optional)
        related_task_id: Task created from this idea (optional)

    Example:
        >>> idea = await Idea.create(
        ...     client=client,
        ...     database_id=db_id,
        ...     title="Add caching layer",
        ...     category="Improvement",
        ...     description="Would improve performance"
        ... )
        >>> await idea.accept()
        >>> task = await idea.create_task(version_id=ver.id)
    """

    def __init__(self, client: "NotionClient", data: dict[str, Any]) -> None:
        """Initialize idea with client and API data."""
        super().__init__(client, data)
        self._idea_cache = client.plugin_cache("ideas")

    # ===== PROPERTIES =====

    @property
    def title(self) -> str:
        """Get idea title."""
        title_prop = self._data["properties"].get("Title") or self._data["properties"].get("title")
        if title_prop and title_prop.get("type") == "title":
            title_data = title_prop.get("title", [])
            if title_data:
                return title_data[0].get("plain_text", "")
        return ""

    @property
    def category(self) -> str:
        """Get idea category."""
        cat_prop = self._data["properties"].get("Category") or self._data["properties"].get("category")
        if cat_prop and cat_prop.get("type") == "select":
            select_data = cat_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def status(self) -> str:
        """Get idea status."""
        status_prop = self._data["properties"].get("Status") or self._data["properties"].get("status")
        if status_prop and status_prop.get("type") == "select":
            select_data = status_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def description(self) -> str:
        """Get idea description."""
        desc_prop = self._data["properties"].get("Description") or self._data["properties"].get("description")
        if desc_prop and desc_prop.get("type") == "rich_text":
            text_data = desc_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def proposed_solution(self) -> str:
        """Get proposed solution."""
        sol_prop = self._data["properties"].get("Proposed Solution") or self._data["properties"].get("proposed_solution")
        if sol_prop and sol_prop.get("type") == "rich_text":
            text_data = sol_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def benefits(self) -> str:
        """Get idea benefits."""
        ben_prop = self._data["properties"].get("Benefits") or self._data["properties"].get("benefits")
        if ben_prop and ben_prop.get("type") == "rich_text":
            text_data = ben_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def effort_estimate(self) -> str:
        """Get effort estimate."""
        effort_prop = self._data["properties"].get("Effort Estimate") or self._data["properties"].get("effort_estimate")
        if effort_prop and effort_prop.get("type") == "select":
            select_data = effort_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def context(self) -> str:
        """Get idea context."""
        ctx_prop = self._data["properties"].get("Context") or self._data["properties"].get("context")
        if ctx_prop and ctx_prop.get("type") == "rich_text":
            text_data = ctx_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def project_id(self) -> str | None:
        """Get related project ID."""
        proj_prop = self._data["properties"].get("Project") or self._data["properties"].get("project")
        if proj_prop and proj_prop.get("type") == "relation":
            relations = proj_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    @property
    def related_task_id(self) -> str | None:
        """Get ID of task created from this idea."""
        task_prop = self._data["properties"].get("Related Task") or self._data["properties"].get("related_task")
        if task_prop and task_prop.get("type") == "relation":
            relations = task_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    # ===== AUTONOMOUS METHODS =====

    @classmethod
    async def get(cls, idea_id: str, *, client: "NotionClient") -> "Idea":
        """Get an idea by ID."""
        cache = client.plugin_cache("ideas")
        if cache and idea_id in cache:
            return cache[idea_id]

        data = await client._api.pages.get(page_id=idea_id)
        idea = cls(client, data)

        if cache:
            cache[idea_id] = idea

        return idea

    @classmethod
    async def create(
        cls,
        *,
        client: "NotionClient",
        database_id: str,
        title: str,
        category: str,
        status: str = "New",
        description: str | None = None,
        proposed_solution: str | None = None,
        benefits: str | None = None,
        effort_estimate: str = "Medium",
        context: str | None = None,
        project_id: str | None = None,
    ) -> "Idea":
        """Create a new idea."""
        from better_notion._api.properties import Title, RichText, Select, Relation

        properties: dict[str, Any] = {
            "Title": Title(content=title),
            "Category": Select(name="Category", value=category),
            "Status": Select(name="Status", value=status),
            "Effort Estimate": Select(name="Effort Estimate", value=effort_estimate),
        }

        if description:
            properties["Description"] = RichText(name="Description", content=description)
        if proposed_solution:
            properties["Proposed Solution"] = RichText(name="Proposed Solution", content=proposed_solution)
        if benefits:
            properties["Benefits"] = RichText(name="Benefits", content=benefits)
        if context:
            properties["Context"] = RichText(name="Context", content=context)
        if project_id:
            properties["Project"] = Relation([project_id])

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        data = await client._api.pages.create(
            parent={"database_id": database_id},
            properties=serialized_properties,
        )

        idea = cls(client, data)

        cache = client.plugin_cache("ideas")
        if cache:
            cache[idea.id] = idea

        return idea

    async def update(
        self,
        *,
        title: str | None = None,
        category: str | None = None,
        status: str | None = None,
        description: str | None = None,
        proposed_solution: str | None = None,
        benefits: str | None = None,
        effort_estimate: str | None = None,
        context: str | None = None,
    ) -> "Idea":
        """Update idea properties."""
        from better_notion._api.properties import Title, RichText, Select

        properties: dict[str, Any] = {}

        if title is not None:
            properties["Title"] = Title(content=title)
        if category is not None:
            properties["Category"] = Select(name="Category", value=category)
        if status is not None:
            properties["Status"] = Select(name="Status", value=status)
        if description is not None:
            properties["Description"] = RichText(name="Description", content=description)
        if proposed_solution is not None:
            properties["Proposed Solution"] = RichText(name="Proposed Solution", content=proposed_solution)
        if benefits is not None:
            properties["Benefits"] = RichText(name="Benefits", content=benefits)
        if effort_estimate is not None:
            properties["Effort Estimate"] = Select(name="Effort Estimate", value=effort_estimate)
        if context is not None:
            properties["Context"] = RichText(name="Context", content=context)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        data = await self._client._api.pages.update(
            page_id=self.id,
            properties=serialized_properties,
        )

        self._data = data

        cache = self._client.plugin_cache("ideas")
        if cache and self.id in cache:
            del cache[self.id]

        return self

    async def delete(self) -> None:
        """Delete the idea."""
        await self._client._api.pages.delete(page_id=self.id)

        cache = self._client.plugin_cache("ideas")
        if cache and self.id in cache:
            del cache[self.id]

    # ===== WORKFLOW METHODS =====

    async def accept(self) -> "Idea":
        """
        Accept this idea (transition to Accepted status).

        Returns:
            Updated Idea instance

        Example:
            >>> await idea.accept()
        """
        return await self.update(status="Accepted")

    async def reject(self, reason: str) -> "Idea":
        """
        Reject this idea (transition to Rejected status).

        Args:
            reason: Rejection reason stored in context

        Returns:
            Updated Idea instance

        Example:
            >>> await idea.reject("Not technically feasible")
        """
        return await self.update(
            status="Rejected",
            context=f"Rejected: {reason}",
        )

    async def defer(self) -> "Idea":
        """
        Defer this idea (transition to Deferred status).

        Returns:
            Updated Idea instance

        Example:
            >>> await idea.defer()
        """
        return await self.update(status="Deferred")

    async def create_task(
        self,
        *,
        version_id: str,
        title: str | None = None,
        task_type: str = "New Feature",
        priority: str = "Medium",
    ) -> Task:
        """
        Create a task from this idea.

        Args:
            version_id: Version to create task in
            title: Task title (defaults to idea title)
            task_type: Type of task
            priority: Task priority

        Returns:
            Created Task instance

        Example:
            >>> task = await idea.create_task(version_id=ver.id, priority="High")
        """
        from better_notion.plugins.official.agents_sdk.models import Task

        # Get database ID
        import json
        from pathlib import Path

        config_path = Path.home() / ".notion" / "workspace.json"
        config = json.loads(config_path.read_text())
        tasks_db_id = config.get("Tasks")

        if not tasks_db_id:
            raise ValueError("Tasks database not found in workspace config")

        task = await Task.create(
            client=self._client,
            database_id=tasks_db_id,
            title=title or self.title,
            version_id=version_id,
            task_type=task_type,
            priority=priority,
        )

        # Link idea to task
        await self.link_to_task(task.id)

        return task

    async def link_to_task(self, task_id: str) -> None:
        """
        Link this idea to a task.

        Args:
            task_id: Task ID to link to

        Example:
            >>> await idea.link_to_task(task.id)
        """
        from better_notion._api.properties import Relation

        await self._client._api.pages.update(
            page_id=self.id,
            properties={
                "Related Task": Relation([task_id]),
            },
        )


class WorkIssue(DatabasePageEntityMixin, BaseEntity):
    """
    Work Issue entity representing a development-time problem.

    Work Issues track blockers, confusions, documentation gaps,
    tooling limitations, and other problems encountered during development.

    Attributes:
        id: Work Issue page ID
        title: Issue title
        project_id: Affected project ID
        task_id: Task where issue occurred (optional)
        type: Issue type (Blocker, Confusion, Documentation, Tooling, etc.)
        severity: Issue severity (Critical, High, Medium, Low)
        status: Issue status (Open, Investigating, Resolved, Won't Fix, Deferred)
        description: What happened
        context: Environment details
        proposed_solution: How to fix
        related_idea_id: Idea this inspired (optional)

    Example:
        >>> issue = await WorkIssue.create(
        ...     client=client,
        ...     database_id=db_id,
        ...     title="API documentation unclear",
        ...     project_id=proj.id,
        ...     type="Documentation",
        ...     severity="Medium"
        ... )
        >>> await issue.resolve("Updated docs with examples")
        >>> idea = await issue.create_idea_from_solution()
    """

    def __init__(self, client: "NotionClient", data: dict[str, Any]) -> None:
        """Initialize work issue with client and API data."""
        super().__init__(client, data)
        self._work_issue_cache = client.plugin_cache("work_issues")

    # ===== PROPERTIES =====

    @property
    def title(self) -> str:
        """Get issue title."""
        title_prop = self._data["properties"].get("Title") or self._data["properties"].get("title")
        if title_prop and title_prop.get("type") == "title":
            title_data = title_prop.get("title", [])
            if title_data:
                return title_data[0].get("plain_text", "")
        return ""

    @property
    def project_id(self) -> str | None:
        """Get affected project ID."""
        proj_prop = self._data["properties"].get("Project") or self._data["properties"].get("project")
        if proj_prop and proj_prop.get("type") == "relation":
            relations = proj_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    @property
    def task_id(self) -> str | None:
        """Get related task ID."""
        task_prop = self._data["properties"].get("Task") or self._data["properties"].get("task")
        if task_prop and task_prop.get("type") == "relation":
            relations = task_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    @property
    def type(self) -> str:
        """Get issue type."""
        type_prop = self._data["properties"].get("Type") or self._data["properties"].get("type")
        if type_prop and type_prop.get("type") == "select":
            select_data = type_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def severity(self) -> str:
        """Get issue severity."""
        sev_prop = self._data["properties"].get("Severity") or self._data["properties"].get("severity")
        if sev_prop and sev_prop.get("type") == "select":
            select_data = sev_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def status(self) -> str:
        """Get issue status."""
        status_prop = self._data["properties"].get("Status") or self._data["properties"].get("status")
        if status_prop and status_prop.get("type") == "select":
            select_data = status_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def description(self) -> str:
        """Get issue description."""
        desc_prop = self._data["properties"].get("Description") or self._data["properties"].get("description")
        if desc_prop and desc_prop.get("type") == "rich_text":
            text_data = desc_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def context(self) -> str:
        """Get issue context."""
        ctx_prop = self._data["properties"].get("Context") or self._data["properties"].get("context")
        if ctx_prop and ctx_prop.get("type") == "rich_text":
            text_data = ctx_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def proposed_solution(self) -> str:
        """Get proposed solution."""
        sol_prop = self._data["properties"].get("Proposed Solution") or self._data["properties"].get("proposed_solution")
        if sol_prop and sol_prop.get("type") == "rich_text":
            text_data = sol_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def related_idea_id(self) -> str | None:
        """Get related idea ID."""
        idea_prop = self._data["properties"].get("Related Idea") or self._data["properties"].get("related_idea")
        if idea_prop and idea_prop.get("type") == "relation":
            relations = idea_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    @property
    def caused_incident_ids(self) -> list[str]:
        """Get incident IDs caused by this work issue."""
        incidents_prop = self._data["properties"].get("Caused Incidents") or self._data["properties"].get("caused_incidents")
        if incidents_prop and incidents_prop.get("type") == "relation":
            relations = incidents_prop.get("relation", [])
            return [r.get("id", "") for r in relations if r.get("id")]
        return []

    @property
    def blocking_task_ids(self) -> list[str]:
        """Get task IDs blocked by this work issue."""
        tasks_prop = self._data["properties"].get("Blocking Tasks") or self._data["properties"].get("blocking_tasks")
        if tasks_prop and tasks_prop.get("type") == "relation":
            relations = tasks_prop.get("relation", [])
            return [r.get("id", "") for r in relations if r.get("id")]
        return []

    # ===== AUTONOMOUS METHODS =====

    @classmethod
    async def get(cls, issue_id: str, *, client: "NotionClient") -> "WorkIssue":
        """Get a work issue by ID."""
        cache = client.plugin_cache("work_issues")
        if cache and issue_id in cache:
            return cache[issue_id]

        data = await client._api.pages.get(page_id=issue_id)
        issue = cls(client, data)

        if cache:
            cache[issue_id] = issue

        return issue

    @classmethod
    async def create(
        cls,
        *,
        client: "NotionClient",
        database_id: str,
        title: str,
        project_id: str,
        task_id: str | None = None,
        type: str = "Blocker",
        severity: str = "Medium",
        status: str = "Open",
        description: str | None = None,
        context: str | None = None,
        proposed_solution: str | None = None,
    ) -> "WorkIssue":
        """Create a new work issue."""
        from better_notion._api.properties import Title, RichText, Select, Relation

        properties: dict[str, Any] = {
            "Title": Title(content=title),
            "Project": Relation([project_id]),
            "Type": Select(name="Type", value=type),
            "Severity": Select(name="Severity", value=severity),
            "Status": Select(name="Status", value=status),
        }

        if task_id:
            properties["Task"] = Relation([task_id])
        if description:
            properties["Description"] = RichText(name="Description", content=description)
        if context:
            properties["Context"] = RichText(name="Context", content=context)
        if proposed_solution:
            properties["Proposed Solution"] = RichText(name="Proposed Solution", content=proposed_solution)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        data = await client._api.pages.create(
            parent={"database_id": database_id},
            properties=serialized_properties,
        )

        issue = cls(client, data)

        cache = client.plugin_cache("work_issues")
        if cache:
            cache[issue.id] = issue

        return issue

    async def update(
        self,
        *,
        title: str | None = None,
        type: str | None = None,
        severity: str | None = None,
        status: str | None = None,
        description: str | None = None,
        context: str | None = None,
        proposed_solution: str | None = None,
    ) -> "WorkIssue":
        """Update work issue properties."""
        from better_notion._api.properties import Title, RichText, Select

        properties: dict[str, Any] = {}

        if title is not None:
            properties["Title"] = Title(content=title)
        if type is not None:
            properties["Type"] = Select(name="Type", value=type)
        if severity is not None:
            properties["Severity"] = Select(name="Severity", value=severity)
        if status is not None:
            properties["Status"] = Select(name="Status", value=status)
        if description is not None:
            properties["Description"] = RichText(name="Description", content=description)
        if context is not None:
            properties["Context"] = RichText(name="Context", content=context)
        if proposed_solution is not None:
            properties["Proposed Solution"] = RichText(name="Proposed Solution", content=proposed_solution)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        data = await self._client._api.pages.update(
            page_id=self.id,
            properties=serialized_properties,
        )

        self._data = data

        cache = self._client.plugin_cache("work_issues")
        if cache and self.id in cache:
            del cache[self.id]

        return self

    async def delete(self) -> None:
        """Delete the work issue."""
        await self._client._api.pages.delete(page_id=self.id)

        cache = self._client.plugin_cache("work_issues")
        if cache and self.id in cache:
            del cache[self.id]

    # ===== WORKFLOW METHODS =====

    async def resolve(self, solution: str) -> "WorkIssue":
        """
        Resolve this issue.

        Args:
            solution: How the issue was resolved

        Returns:
            Updated WorkIssue instance

        Example:
            >>> await issue.resolve("Updated documentation")
        """
        return await self.update(
        status="Resolved",
        proposed_solution=solution,
    )

    async def investigate(self) -> "WorkIssue":
        """
        Mark issue as under investigation.

        Returns:
            Updated WorkIssue instance

        Example:
            >>> await issue.investigate()
        """
        return await self.update(status="Investigating")

    async def link_to_idea(self, idea_id: str) -> None:
        """
        Link this issue to an idea.

        Args:
            idea_id: Idea ID to link to

        Example:
            >>> await issue.link_to_idea(idea.id)
        """
        from better_notion._api.properties import Relation

        await self._client._api.pages.update(
            page_id=self.id,
            properties={
                "Related Idea": Relation([idea_id]),
            },
        )

    async def create_idea_from_solution(self) -> Idea:
        """
        Create an idea from this issue's solution.

        Returns:
            Created Idea instance

        Example:
            >>> idea = await issue.create_idea_from_solution()
        """
        from better_notion.plugins.official.agents_sdk.models import Idea

        import json
        from pathlib import Path

        config_path = Path.home() / ".notion" / "workspace.json"
        config = json.loads(config_path.read_text())
        ideas_db_id = config.get("Ideas")

        if not ideas_db_id:
            raise ValueError("Ideas database not found in workspace config")

        idea = await Idea.create(
            client=self._client,
            database_id=ideas_db_id,
            title=f"Prevent issue: {self.title}",
            category="Process",
            status="New",
            description=f"Issue {self.id} revealed process gap",
            proposed_solution=self.proposed_solution or "Implement prevention",
            context=self.context,
            project_id=self.project_id,
        )

        # Link issue to idea
        await self.link_to_idea(idea.id)

        return idea

    async def caused_incidents(self) -> list["Incident"]:
        """
        Get incidents caused by this work issue.

        Returns:
            List of Incident instances

        Example:
            >>> incidents = await issue.caused_incidents()
        """
        from better_notion.plugins.official.agents_sdk.models import Incident

        incidents = []
        for incident_id in self.caused_incident_ids:
            try:
                incident = await Incident.get(incident_id, client=self._client)
                incidents.append(incident)
            except Exception:
                pass
        return incidents

    async def blocking_tasks(self) -> list["Task"]:
        """
        Get tasks blocked by this work issue.

        Returns:
            List of Task instances

        Example:
            >>> tasks = await issue.blocking_tasks()
        """
        from better_notion.plugins.official.agents_sdk.models import Task

        tasks = []
        for task_id in self.blocking_task_ids:
            try:
                task = await Task.get(task_id, client=self._client)
                tasks.append(task)
            except Exception:
                pass
        return tasks


class Incident(DatabasePageEntityMixin, BaseEntity):
    """
    Incident entity representing a production incident.

    Incidents track production problems separate from development
    work issues. Critical for production reliability and MTTR tracking.

    Attributes:
        id: Incident page ID
        title: Incident title
        project_id: Affected project ID
        affected_version_id: Version where incident occurred
        severity: Incident severity (Critical, High, Medium, Low)
        type: Incident type (Bug, Crash, Performance, Security, etc.)
        status: Incident status (Open, Investigating, Fix in Progress, Resolved)
        fix_task_id: Task to fix this incident (optional)
        root_cause: Analysis of what went wrong
        discovery_date: When incident was discovered
        resolved_date: When incident was resolved (optional)

    Example:
        >>> incident = await Incident.create(
        ...     client=client,
        ...     database_id=db_id,
        ...     title="Production database down",
        ...     project_id=proj.id,
        ...     affected_version_id=ver.id,
        ...     severity="Critical",
        ...     type="Outage"
        ... )
        >>> task = await incident.create_fix_task(version_id=hotfix.id)
        >>> await incident.resolve("Config error in connection pool")
        >>> mttr = incident.calculate_mttr()
    """

    def __init__(self, client: "NotionClient", data: dict[str, Any]) -> None:
        """Initialize incident with client and API data."""
        super().__init__(client, data)
        self._incident_cache = client.plugin_cache("incidents")

    # ===== PROPERTIES =====

    @property
    def title(self) -> str:
        """Get incident title."""
        title_prop = self._data["properties"].get("Title") or self._data["properties"].get("title")
        if title_prop and title_prop.get("type") == "title":
            title_data = title_prop.get("title", [])
            if title_data:
                return title_data[0].get("plain_text", "")
        return ""

    @property
    def project_id(self) -> str | None:
        """Get affected project ID."""
        proj_prop = self._data["properties"].get("Project") or self._data["properties"].get("project")
        if proj_prop and proj_prop.get("type") == "relation":
            relations = proj_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    @property
    def affected_version_id(self) -> str | None:
        """Get affected version ID."""
        ver_prop = self._data["properties"].get("Affected Version") or self._data["properties"].get("affected_version")
        if ver_prop and ver_prop.get("type") == "relation":
            relations = ver_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    @property
    def severity(self) -> str:
        """Get incident severity."""
        sev_prop = self._data["properties"].get("Severity") or self._data["properties"].get("severity")
        if sev_prop and sev_prop.get("type") == "select":
            select_data = sev_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def type(self) -> str:
        """Get incident type."""
        type_prop = self._data["properties"].get("Type") or self._data["properties"].get("type")
        if type_prop and type_prop.get("type") == "select":
            select_data = type_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def status(self) -> str:
        """Get incident status."""
        status_prop = self._data["properties"].get("Status") or self._data["properties"].get("status")
        if status_prop and status_prop.get("type") == "select":
            select_data = status_prop.get("select")
            if select_data:
                return select_data.get("name", "Unknown")
        return "Unknown"

    @property
    def fix_task_id(self) -> str | None:
        """Get fix task ID."""
        task_prop = self._data["properties"].get("Fix Task") or self._data["properties"].get("fix_task")
        if task_prop and task_prop.get("type") == "relation":
            relations = task_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    @property
    def root_cause(self) -> str:
        """Get root cause analysis."""
        rc_prop = self._data["properties"].get("Root Cause") or self._data["properties"].get("root_cause")
        if rc_prop and rc_prop.get("type") == "rich_text":
            text_data = rc_prop.get("rich_text", [])
            if text_data:
                return text_data[0].get("plain_text", "")
        return ""

    @property
    def discovery_date(self) -> str | None:
        """Get discovery date."""
        date_prop = self._data["properties"].get("Discovery Date") or self._data["properties"].get("discovery_date")
        if date_prop and date_prop.get("type") == "date":
            date_data = date_prop.get("date")
            if date_data and date_data.get("start"):
                return date_data["start"]
        return None

    @property
    def resolved_date(self) -> str | None:
        """Get resolved date."""
        date_prop = self._data["properties"].get("Resolved Date") or self._data["properties"].get("resolved_date")
        if date_prop and date_prop.get("type") == "date":
            date_data = date_prop.get("date")
            if date_data and date_data.get("start"):
                return date_data["start"]
        return None

    @property
    def root_cause_work_issue_id(self) -> str | None:
        """Get the work issue ID that caused this incident."""
        issue_prop = self._data["properties"].get("Root Cause Work Issue") or self._data["properties"].get("root_cause_work_issue")
        if issue_prop and issue_prop.get("type") == "relation":
            relations = issue_prop.get("relation", [])
            if relations:
                return relations[0].get("id")
        return None

    # ===== AUTONOMOUS METHODS =====

    @classmethod
    async def get(cls, incident_id: str, *, client: "NotionClient") -> "Incident":
        """Get an incident by ID."""
        cache = client.plugin_cache("incidents")
        if cache and incident_id in cache:
            return cache[incident_id]

        data = await client._api.pages.get(page_id=incident_id)
        incident = cls(client, data)

        if cache:
            cache[incident_id] = incident

        return incident

    @classmethod
    async def create(
        cls,
        *,
        client: "NotionClient",
        database_id: str,
        title: str,
        project_id: str,
        affected_version_id: str,
        severity: str = "High",
        type: str = "Bug",
        status: str = "Open",
        discovery_date: str | None = None,
    ) -> "Incident":
        """Create a new incident."""
        from datetime import datetime
        from better_notion._api.properties import Title, Date, Select, Relation

        properties: dict[str, Any] = {
            "Title": Title(content=title),
            "Project": Relation([project_id]),
            "Affected Version": Relation([affected_version_id]),
            "Severity": Select(name="Severity", value=severity),
            "Type": Select(name="Type", value=type),
            "Status": Select(name="Status", value=status),
            "Discovery Date": Date(name="Discovery Date", value=discovery_date or datetime.now().isoformat()),
        }

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        data = await client._api.pages.create(
            parent={"database_id": database_id},
            properties=serialized_properties,
        )

        incident = cls(client, data)

        cache = client.plugin_cache("incidents")
        if cache:
            cache[incident.id] = incident

        return incident

    async def update(
        self,
        *,
        title: str | None = None,
        severity: str | None = None,
        status: str | None = None,
        root_cause: str | None = None,
        resolved_date: str | None = None,
    ) -> "Incident":
        """Update incident properties."""
        from better_notion._api.properties import Title, RichText, Select, Date

        properties: dict[str, Any] = {}

        if title is not None:
            properties["Title"] = Title(content=title)
        if severity is not None:
            properties["Severity"] = Select(name="Severity", value=severity)
        if status is not None:
            properties["Status"] = Select(name="Status", value=status)
        if root_cause is not None:
            properties["Root Cause"] = RichText(name="Root Cause", content=root_cause)
        if resolved_date is not None:
            properties["Resolved Date"] = Date(name="Resolved Date", value=resolved_date)

        # Convert Property objects to dicts for API
        serialized_properties = {
            key: prop.to_dict() if hasattr(prop, 'to_dict') else prop
            for key, prop in properties.items()
        }

        data = await self._client._api.pages.update(
            page_id=self.id,
            properties=serialized_properties,
        )

        self._data = data

        cache = self._client.plugin_cache("incidents")
        if cache and self.id in cache:
            del cache[self.id]

        return self

    async def delete(self) -> None:
        """Delete the incident."""
        await self._client._api.pages.delete(page_id=self.id)

        cache = self._client.plugin_cache("incidents")
        if cache and self.id in cache:
            del cache[self.id]

    # ===== WORKFLOW METHODS =====

    async def investigate(self) -> "Incident":
        """
        Mark incident as under investigation.

        Returns:
            Updated Incident instance

        Example:
            >>> await incident.investigate()
        """
        return await self.update(status="Investigating")

    async def assign(self, task_id: str) -> "Incident":
        """
        Assign a fix task to this incident.

        Args:
            task_id: Task ID that will fix this incident

        Returns:
            Updated Incident instance

        Example:
            >>> await incident.assign(task.id)
        """
        from better_notion._api.properties import Relation

        await self._client._api.pages.update(
            page_id=self.id,
            properties={
                "Fix Task": Relation([task_id]).to_dict(),
                "Status": Select(name="Status", value="Fix in Progress").to_dict(),
            },
        )

        # Update local data
        self._data["properties"]["Fix Task"] = {"type": "relation", "relation": [{"id": task_id}]}
        self._data["properties"]["Status"] = {"type": "select", "select": {"name": "Fix in Progress"}}

        return self

    async def resolve(self, root_cause: str, resolved_date: str | None = None) -> "Incident":
        """
        Resolve this incident.

        Args:
            root_cause: Analysis of what went wrong
            resolved_date: When incident was resolved (defaults to now)

        Returns:
            Updated Incident instance

        Example:
            >>> await incident.resolve("Config error in connection pool")
        """
        from datetime import datetime

        return await self.update(
            status="Resolved",
        root_cause=root_cause,
        resolved_date=resolved_date or datetime.now().isoformat(),
        )

    def calculate_mttr(self) -> float | None:
        """
        Calculate Mean Time To Resolve in minutes.

        Returns:
            MTTR in minutes, or None if not resolved

        Example:
            >>> mttr_minutes = incident.calculate_mttr()
            >>> print(f"MTTR: {mttr_minutes:.1f} minutes")
        """
        if not self.resolved_date or not self.discovery_date:
            return None

        from datetime import datetime

        resolved = datetime.fromisoformat(self.resolved_date)
        discovered = datetime.fromisoformat(self.discovery_date)

        mttr_seconds = (resolved - discovered).total_seconds()
        return mttr_seconds / 60

    async def create_fix_task(
        self,
        *,
        version_id: str,
        title: str | None = None,
        priority: str = "Critical",
    ) -> Task:
        """
        Create a fix task for this incident.

        Args:
            version_id: Version to create fix in
            title: Task title (defaults to incident title)
            priority: Task priority

        Returns:
            Created Task instance

        Example:
            >>> task = await incident.create_fix_task(
            ...     version_id=hotfix.id,
            ...     priority="Critical"
            ... )
        """
        from better_notion.plugins.official.agents_sdk.models import Task

        import json
        from pathlib import Path

        config_path = Path.home() / ".notion" / "workspace.json"
        config = json.loads(config_path.read_text())
        tasks_db_id = config.get("Tasks")

        if not tasks_db_id:
            raise ValueError("Tasks database not found in workspace config")

        task = await Task.create(
            client=self._client,
            database_id=tasks_db_id,
            title=title or f"Fix incident: {self.title}",
            version_id=version_id,
            task_type="Bug Fix",
            priority=priority,
        )

        # Assign task to incident
        await self.assign(task.id)

        return task

    async def root_cause_work_issue(self) -> "WorkIssue | None":
        """
        Get the work issue that caused this incident.

        Returns:
            WorkIssue instance or None

        Example:
            >>> issue = await incident.root_cause_work_issue()
        """
        from better_notion.plugins.official.agents_sdk.models import WorkIssue

        issue_id = self.root_cause_work_issue_id
        if not issue_id:
            return None

        try:
            return await WorkIssue.get(issue_id, client=self._client)
        except Exception:
            return None

    async def link_to_work_issue(self, work_issue_id: str) -> None:
        """
        Link this incident to a work issue (root cause).

        Args:
            work_issue_id: Work issue ID to link to

        Example:
            >>> await incident.link_to_work_issue(issue_id)
        """
        from better_notion._api.properties import Relation

        await self._client._api.pages.update(
            page_id=self.id,
            properties={
                "Root Cause Work Issue": Relation([work_issue_id]).to_dict(),
            },
        )

        # Update local data
        self._data["properties"]["Root Cause Work Issue"] = {
            "type": "relation",
            "relation": [{"id": work_issue_id}]
        }

    async def unlink_work_issue(self) -> None:
        """
        Unlink this incident from its work issue.

        Example:
            >>> await incident.unlink_work_issue()
        """
        from better_notion._api.properties import Relation

        await self._client._api.pages.update(
            page_id=self.id,
            properties={
                "Root Cause Work Issue": Relation([]).to_dict(),
            },
        )

        # Update local data
        self._data["properties"]["Root Cause Work Issue"] = {
            "type": "relation",
            "relation": []
        }
